<?php
/*
Plugin Name: LJ Longtail SEO
Plugin URI: http://www.thelazysysadmin.net/software/wordpress-plugins/lj-longtail-seo/
Description: LJ Longtail SEO is a tool that detects search engine visits and uses this information to display a list of links based on second page search results
Author: Jon Smith
Version: 1.0
Author URI: http://www.thelazysysadmin.net/
*/

class LJLongtailSEO {

  private $dbversion = "1.75";
  private $pluginversion = "1.0";

  private $defaults = array
    (
      'enabled' => true,
      'widget-title' => 'Popular Searches',
      'widget-numitems' => '5',
      'daysbeforearchive' => '28',
      'showdonate' => false
    );

  function LJLongtailSEO() {
    add_action('LJLongtailSEO_cron_hook', array(&$this, 'cron'));
    add_action('admin_menu', array(&$this, 'admin_menu'));

    add_filter("wp_head", array(&$this, "checkreferrer"));

    register_sidebar_widget("LJ Longtail SEO", array(&$this, "widget"));
    register_widget_control("LJ Longtail SEO", array(&$this, "widget_control"));

    register_activation_hook(__FILE__, array(&$this, 'db_install'));
    register_deactivation_hook(__FILE__, array(&$this, 'plugin_deactivate'));                                        
  }

  function admin_menu() {
    if ( function_exists('add_options_page') )
      add_options_page(__('LJ Longtail SEO Configuration'), __('LJ Longtail SEO'), 8, __FILE__, array(&$this, 'config'));
  }

  function checkreferrer() {
    global $wpdb, $wp_query;
    
    if (is_feed())
      return;
    if (is_category())
      return; 
    if (is_tag())
      return;
    if (is_front_page())
      return;   
    
    $table_name = $wpdb->prefix."ljlongtailseo";

    if (isset($_SERVER["HTTP_REFERER"])) {
      $arraylist = parse_url($_SERVER['HTTP_REFERER']);    
      
      if ($this->is_search($arraylist)) {
        $query = $this->get_query($arraylist);
        if ($query != false) {
          $query = urldecode($query);
          $query = preg_replace('/[^a-zA-Z0-9-\/. ]/', '', $query);
          $query = ereg_replace("[ \t\n\r]+", " ", $query);
          $query = strtolower($query);
          $serpposition = $this->get_serpposition($arraylist);
          $postid = $wp_query->post->ID;
          $wpdb->query($wpdb->prepare("INSERT INTO $table_name (postid, serppage, query, referrer) VALUES (%d, %d, %s, %s);", $postid, $serpposition, $query, $_SERVER["HTTP_REFERER"])); 
        }
      }
    }
  }
  
  function is_search($urlparts) {
    $urllist[] = "bing.com";
    $urllist[] = ".google.";
    $urllist[] = ".yahoo.";
    $urllist[] = ".ask.";
    
    foreach ($urllist as $searchurl) {
      if (stripos($urlparts["host"], $searchurl) !== false) {
        return true;
      } 
    }
    
    return false;
  } 
  
  function get_query($urlparts) {
    if (stripos($urlparts["query"], "q=") !== false) {
      $querylist = explode("&", $urlparts["query"]);
      
      foreach ($querylist as $query) {
        if (stripos($query, "q=") !== false) {
          $t = explode("=", $query);
          
          if (stripos($t[1], "site:") !== false) return false;
          if (stripos($t[1], "related:") !== false) return false;
          if (stripos($t[1], "cache:") !== false) return false;
          if (stripos($t[1], "link:") !== false) return false;
          
          return $t[1];
        } 
      }
    }
    
    return false; 
  }
  
  function get_serpposition($urlparts) {
    $querypart[] = "first";
    $querypart[] = "page";
    $querypart[] = "b";
    $querypart[] = "start";
    $querypart[] = "cd";
    
    $querylist = explode("&", $urlparts["query"]);
    
    foreach ($querypart as $part) {
      if (stripos($urlparts["query"], $part) !== false) {
        switch ($part) {
          case "first":
          case "b":
          case "start":
          case "cd":
            if (preg_match("/[&\?](first|b|start|cd)=([0-9]*)/i", $urlparts["query"], $temp)){
              return (floor($temp[2] / 10) + 1);
            }
            break;
          case "page":
            if (preg_match("/[&\?](page)=([0-9]*)/i", $urlparts["query"], $temp)){
              return $temp[2];
            }
            break; 
        }
      } 
    }
    
    return -1; 
  }
  
  function widget($args) {
    global $wpdb, $wp_locale;

    extract($args);

    $options = get_option('LJLongtailSEO');
    $table_name = $wpdb->prefix."ljlongtailseo";

    if ($options === false) {
      $options = array();
    }

    foreach ($this->defaults as $key => $value) {
      if (!isset ($options[$key]))
        $options[$key] = $value;
    }

    echo "\n<!-- LJLongtailSEO Version ".$this->pluginversion." Start -->\n";
    echo $before_widget;
    echo "<h3>".$options['widget-title']."</h3>\n";

    $dbresults = $wpdb->get_results($wpdb->prepare("SELECT postid, query, serppage, count(query) AS num, archive FROM $table_name GROUP BY postid, query, serppage HAVING serppage != -1 AND serppage != 1 AND postid != 0 AND archive = 0 ORDER BY serppage, num DESC LIMIT 0, %d;", $options['widget-numitems']), ARRAY_A);
    
    if (count($dbresults) > 0) {
      echo "  <ul>\n";
      
      foreach ($dbresults as $result) {
        echo "    <li><a href=\"".get_permalink($result["postid"])."\" title=\"".$result["query"]."\">".$result["query"]."</a></li>\n"; 
      }
      
      echo "  </ul>\n";
    } else {
      echo "There are currently no popular searches registered. Please be patient, in the meantime it is probably best to disable the widget. Check on the widget results in the admin screen."; 
    }
    
    echo $after_widget;
    echo "\n<!-- LJLongtailSEO End -->\n";
  }

  function widget_control() {

    $options = get_option('LJLongtailSEO');

    if ($options === false) {
      $options = array();
    }

    foreach ($this->defaults as $key => $value) {
      if (!isset ($options[$key]))
        $options[$key] = $value;
    }

    if ($_POST['widget-LJLongtailSEO-submit']) {
      $options['widget-title'] = $_POST['widget-LJLongtailSEO-title'];
      $options['widget-numitems'] = sprintf("%d", $_POST['widget-LJLongtailSEO-numitems']);
      
      update_option('LJLongtailSEO', $options);
    }

    echo '<p><label for="widget-LJLongtailSEO-title">Title: <br />';
    echo '<input type="text" class="widefat" id="widget-LJLongtailSEO-title" name="widget-LJLongtailSEO-title" value="'.$options['widget-title'].'" /></label></p>';

    echo '<p><label for="widget-LJLongtailSEO-numitems">Num of Items: <br />';
    echo '<input type="text" class="widefat" id="widget-LJLongtailSEO-numitems" name="widget-LJLongtailSEO-numitems" value="'.$options['widget-numitems'].'" /></label></p>';

    echo '<input type="hidden" id="widget-LJLongtailSEO-submit" name="widget-LJLongtailSEO-submit" value="1" />';
  }
  
  function cron() {
    global $wpdb;
    
    $options = get_option('LJLongtailSEO');
    $table_name = $wpdb->prefix."ljlongtailseo";

    if ($options === false) {
      $options = array();
    }

    foreach ($this->defaults as $key => $value) {
      if (!isset ($options[$key]))
        $options[$key] = $value;
    }

    $sql = "UPDATE $table_name SET archive = 1 ";
    $sql .= "WHERE (UNIX_TIMESTAMP() - UNIX_TIMESTAMP(timestamp)) > %d;";
    
    $sql = $wpdb->prepare($sql, ($options['daysbeforearchive'] * 86400));
  }
  
  function config() {
    global $wpdb;

    $options = get_option('LJLongtailSEO');
    $table_name = $wpdb->prefix."ljlongtailseo";

    if ($options === false) {
      $options = array();
    }

    foreach ($this->defaults as $key => $value) {
      if (!isset ($options[$key]))
        $options[$key] = $value;
    }

    $action = isset($_REQUEST['action']) ? $_REQUEST['action'] : '';

    switch ($action) {
      case "editsettings":
        check_admin_referer('LJongtailSEO-editsettings');

        $links_archivedays = isset($_REQUEST['linksarchivedays']) ? $_REQUEST['linksarchivedays'] : 28;
        $links_donate = isset($_REQUEST['linksdonate']) ? $_REQUEST['linksdonate'] : 0;

        $options['daysbeforearchive'] = sprintf("%d", $links_archivedays);
        $options['showdonate'] = ($links_donate == 1) ? true : false;
        
        if ($options['daysbeforearchive'] == 0) {
          $options['daysbeforearchive'] = 28;
        }

        break;
      case "maintenance":
        check_admin_referer('LJongtailSEO-maintenance');
        
        if (isset($_REQUEST["emptydb"])) {
          $wpdb->query("DELETE FROM $table_name;");
        }
        
        if (isset($_REQUEST["cleandb"])) {
          $wpdb->query("DELETE FROM $table_name WHERE archive = 1;");
        }
        
        break; 
    }
?>
<div class='wrap'>
<h2>LJ Longtail SEO</h2>
<h3>Top 10 Search Queries (includes Page 1 results, excludes archived records)</h3>
<div class="clear"></div>
<table class="widefat tag fixed" cellspacing="0">
  <thead>
  <tr>
  <th scope="col" id="position" class="manage-column column-position" style="">Position</th>
  <th scope="col" id="query" class="manage-column column-query" style="">Query</th>
  <th scope="col" id="postfound" class="manage-column column-postfound" style="">Post Found</th>
  <th scope="col" id="hits" class="manage-column column-hits" style="">Hits</th>
  </tr>
  </thead>

  <tfoot>
  <tr>
  <th scope="col" id="position" class="manage-column column-position" style="">Position</th>
  <th scope="col" id="query" class="manage-column column-query" style="">Query</th>
  <th scope="col" id="postfound" class="manage-column column-postfound" style="">Post Found</th>
  <th scope="col" id="hits" class="manage-column column-hits" style="">Hits</th>
  </tr>
  </tfoot>

  <tbody id="the-list" class="list:tag">
<?php
  
  $results = $wpdb->get_results("SELECT postid, query, count(query) AS num, archive FROM $table_name GROUP BY postid, query HAVING archive = 0 ORDER BY num DESC LIMIT 0, 10;", ARRAY_A);
  
  if (count($results) > 0) {
    $i = 1;
    foreach ($results as $result) {
?>
    <tr id="tag-<?php echo $i; ?>" class="iedit">
      <td class="text column-position"><?php echo $i; ?></td>
      <td class="text column-query"><?php echo $result['query']; ?></td>
      <td class="text column-postfound"><?php echo get_the_title($result['postid']); ?></td>
      <td class="text column-hits"><?php echo $result['num']; ?></td>
    </tr>
<?php 
      $i++;    
    }
  }
?>
</tbody>

</table>
<h3>Links that will be shown in widget</h3>
<div class="clear"></div>
<table class="widefat tag fixed" cellspacing="0">
  <thead>
  <tr>
  <th scope="col" id="query" class="manage-column column-query" style="">Query</th>
  <th scope="col" id="postfound" class="manage-column column-postfound" style="">Post Found</th>
  <th scope="col" id="hits" class="manage-column column-hits" style="">Hits</th>
  </tr>
  </thead>

  <tfoot>
  <tr>
  <th scope="col" id="query" class="manage-column column-query" style="">Query</th>
  <th scope="col" id="postfound" class="manage-column column-postfound" style="">Post Found</th>
  <th scope="col" id="hits" class="manage-column column-hits" style="">Hits</th>
  </tr>
  </tfoot>

  <tbody id="the-list" class="list:tag">
<?php
  
  $results = $wpdb->get_results($wpdb->prepare("SELECT postid, query, serppage, count(query) AS num, archive FROM $table_name GROUP BY postid, query, serppage HAVING serppage != -1 AND serppage != 1 AND postid != 0 AND archive = 0 ORDER BY serppage, num DESC LIMIT 0, %d;", $options['widget-numitems']), ARRAY_A);

  if (count($results) > 0) {
    foreach ($results as $result) {
?>
    <tr id="tag-<?php echo $i; ?>" class="iedit">
      <td class="text column-query"><?php echo $result['query']; ?></td>
      <td class="text column-postfound"><?php echo get_the_title($result['postid']); ?></td>
      <td class="text column-hits"><?php echo $result['num']; ?></td>
    </tr>
<?php 
    }
  }  
?>
</tbody>

</table>

<h3>Maintenance</h3>
<div class="clear"></div>
<form action="" method="post" id="LJLongtailSEOEditForm" accept-charset="utf-8">
  <input type="hidden" name="action" value="maintenance" />
  <?php wp_nonce_field('LJongtailSEO-maintenance'); ?>
  Running Clean Database will delete all records in the LJ Longtail SEO database marked as Archived. This operation cannot be reversed.
  <p class="submit"><input onclick="return confirm('Are you sure you want to Clean the database?');" type="submit" class="button-primary" name="cleandb" value="Clean Database" /></p>
  Running Empty Database will delete ALL RECORDS IN THE DATABASE. This operation cannot be reversed. Unless of course you have backups, you do have backups dont you.
  <p class="submit"><input onclick="return confirm('Are you sure you want to Empty the database?');" type="submit" class="button-primary" name="emptydb" value="Empty Database" /></p>
</form>

<h3>Settings</h3>
<div class="clear"></div>
<form action="" method="post" id="LJLongtailSEOEditForm" accept-charset="utf-8">
  <input type="hidden" name="action" value="editsettings" />
  <?php wp_nonce_field('LJongtailSEO-editsettings'); ?>
  <table class="form-table">
  <tr class="form-field form-required">
    <th scope="row" valign="top"><label for="linkurl">Days Befores Archiving:</label></th>

    <td><input name="linksarchivedays" id="linksarchivedays" type="text" value="<?php echo $options['daysbeforearchive']; ?>" size="40" aria-required="true" /><br />
          <span class="description">Number of days before database records get marked as archived. Records marked as archive are not used for the widget display.</span></td>
  </tr>
  <tr valign="top">
  <th scope="row">Show me the donate button</th>
  <td><fieldset><legend class="screen-reader-text"><span>Show the donate button</span></legend>
  <label for="linksdonate">
  <input name="linksdonate" type="checkbox" id="linksdonate" value="1" <?php if ($options['showdonate']) echo "checked='checked'"; ?> />
  If you like this plugin and would like to support it please turn this option on to show the donate button.</label><br />
  </fieldset></td>
  </tr>
</table>
  <p class="submit"><input type="submit" class="button-primary" name="submit" value="Update Settings" /></p>
</form>
<?php
    if ($options['showdonate']) {
?>
<div align="right">
If you would like to make a donation,<br />
please feel free to submit feature requests<br />
for this plugin via <a href="http://www.thelazysysadmin.net/software/wordpress-plugins/lj-longtail-seo/" target="_blank">LJ Longtail SEO</a><br />
plugin page. Submit a feature request even if<br />
you dont make a donation. Requests with donations will<br />
be given a higher priority :-).
<form action="https://www.paypal.com/cgi-bin/webscr" method="post">
<input type="hidden" name="cmd" value="_s-xclick">
<input type="hidden" name="hosted_button_id" value="8087129">
<input type="image" src="https://www.paypal.com/en_AU/i/btn/btn_donateCC_LG.gif" border="0" name="submit" alt="PayPal - The safer, easier way to pay online.">
<img alt="" border="0" src="https://www.paypal.com/en_AU/i/scr/pixel.gif" width="1" height="1">
</form>
</div>
</div>
<?php
    }
    
    update_option('LJLongtailSEO', $options);

  }
  
  function db_install() {
    global $wpdb;

    $currentdbversion = get_option('ljlongtailseo-dbversion');

    $table_name = $wpdb->prefix."ljlongtailseo";

    if ( ($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name) || ($currentdbversion != $this->dbversion)) {
      $sql = "CREATE TABLE $table_name (
        id INT NOT NULL AUTO_INCREMENT  PRIMARY KEY ,
        postid INT NOT NULL ,
        query VARCHAR(255) NOT NULL ,
        serppage INT NOT NULL ,
        referrer TEXT NOT NULL ,
        timestamp TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ,
        archive TINYINT NOT NULL DEFAULT '0'
      );";

      require_once(ABSPATH.'wp-admin/includes/upgrade.php');
      dbDelta($sql);
      
      update_option("ljlongtailseo-dbversion", $this->dbversion);
    }

    $options = get_option('LJLongtailSEO');

    if ($options === false) {
      $options = array();
    }

    foreach ($this->defaults as $key => $value) {
      if (!isset ($options[$key]))
        $options[$key] = $value;
    }
    
    wp_schedule_event(time(), 'daily', 'LJLongtailSEO_cron_hook'); 

    update_option('LJLongtailSEO', $options);
  }
  
  function plugin_deactivate() {
    wp_clear_scheduled_hook('LJLongtailSEO_cron_hook'); 
  }

}

$LJLongtailSEO = new LJLongtailSEO();

?>